import click, os, json
from toolbox.logger import Log
from .vaultclient import VAULTclient
from .secrets import secrets
from .keys import keys
from tabulate import tabulate
from configstore.configstore import Config
from toolbox.misc import set_terminal_width
from .iam import get_latest_profile
from toolbox.menumaker import Menu
from datetime import datetime
from datetime import timedelta

CONFIG = Config('ocitools')
IGNORE = ['latest']

@click.group('vault', invoke_without_command=True, help='module to manage vaults', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform vault actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def vault(ctx, menu):
    profile_name = ctx.obj['PROFILE']
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False
    pass

@vault.command(help='manually refresh vaults stored in cache', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def refresh(ctx):
    profile_name = ctx.obj['PROFILE']
    _refresh('cached_vaults', profile_name)

def _refresh(cache_type, profile_name):
    try:
        if cache_type == 'cached_vaults':
            VAULT = get_VAULTclient(profile_name, auto_refresh=False)
            RESULT = VAULT.refresh('cached_vaults', profile_name)
        return True
    except:
        False

@vault.command(help='create OCI vault for region and tenant', context_settings={'help_option_names':['-h','--help']})
@click.argument('vault', required=True)
@click.pass_context
def create(ctx, vault):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _create(ctx, profile_name, oci_region, vault)

def _create(ctx, profile_name, oci_region, vault):
    VAULT = get_VAULTclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in IGNORE:
            continue
        CACHED_COMPARTMENTS = VAULT.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        OCID = CHOICE.split('\t')[0].strip()
        NAME = CHOICE.split('\t')[1].strip()
    else:
        Log.critical('please choose a compartment')

    Log.info(f'creating vault {vault} in compartment {NAME}')
    RESPONSE = VAULT.create_vault(OCID, vault)
    if RESPONSE:
        Log.info(f'{vault} has been created in compartment {NAME}')
    else:
        Log.critical(f'problem while creating vault {vault}')
    
@vault.command(help='menu-guided solution to delete OCI vault\'s for region and tenant', context_settings={'help_option_names':['-h','--help']})
@click.option('-a', '--all', help='delete all vaults found', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def delete(ctx):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _delete(ctx, profile_name, oci_region, all)

def _delete(ctx, profile_name, oci_region, all):
    VAULT = get_VAULTclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in IGNORE:
            continue
        CACHED_COMPARTMENTS = VAULT.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        OCID = CHOICE.split('\t')[0].strip()
        NAME = CHOICE.split('\t')[1].strip()
    else:
        Log.critical('please choose a compartment')
    VAULTS = VAULT.get_vaults(OCID)
    DATA = []
    DAYS = datetime.now() + timedelta(days=15)
    for MYVAULT in VAULTS:
        if MYVAULT.lifecycle_state not in 'ACTIVE':
            continue
        VAULT_NAME = MYVAULT.display_name.ljust(50)
        OCID = MYVAULT.id.ljust(100)
        DATA.append(OCID + '\t' + VAULT_NAME)
    if all is not True:
        INPUT = f'Vaults => {profile_name}'
        CHOICE = runMenu(DATA, INPUT)
        if CHOICE:
            CHOICE = ''.join(CHOICE)
            OCID = CHOICE.split('\t')[0].strip()
            VAULT_NAME = CHOICE.split('\t')[1].strip()
        else:
            Log.critical('please choose a vault for deletion')
        Log.info(f'scheduling deletion of vault {VAULT_NAME} in compartment {NAME}')
        RESPONSE = VAULT.delete_vault(OCID, DAYS)
        if RESPONSE:
            Log.info(f'{VAULT_NAME} is now pending deletion on {DAYS}')
        else:
            Log.critical(f'problem while scheduling the deletion of {VAULT_NAME}')
    else:
        for MYVAULT in VAULTS:
            if MYVAULT.lifecycle_state not in 'ACTIVE':
                continue
            VAULT_NAME = MYVAULT.display_name.ljust(50)
            OCID = MYVAULT.id.ljust(100)
            Log.info(f'scheduling deletion of vault {VAULT_NAME} in compartment {NAME}')
            RESPONSE = VAULT.delete_vault(OCID, DAYS)
            if RESPONSE:
                Log.info(f'{VAULT_NAME} is now pending deletion on {DAYS}')
            else:
                Log.critical(f'problem while scheduling the deletion of {VAULT_NAME}')

@vault.command(help='show the data stored in cached vault', context_settings={'help_option_names':['-h','--help']})
@click.argument('vault', required=False)
@click.pass_context
def show(ctx, vault):
    profile_name = ctx.obj['PROFILE']
    _show(ctx, profile_name, vault)

def _show(ctx, profile_name, vault):
    if not vault:
        if ctx.obj["MENU"]:
            DATA = []
            DICT = {}
            VAULT = get_VAULTclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = VAULT.get_cached_vaults(profile_name)
            for I in RESPONSE:
                if I not in 'last_cache_update':
                    NAME = RESPONSE[I]['display_name']
                    DATA.append(I + '\t' + NAME)
            INPUT = 'Vault Manager'
            CHOICE = runMenu(DATA, INPUT)
            try:
                CHOICE = ''.join(CHOICE)
                OCID = CHOICE.split('\t')[0].strip()
            except:
                Log.critical("please select a vault to continue...")
            RESPONSE = VAULT.describe(OCID, profile_name)
            Log.info(f"describing {OCID}:\n")
            for I in RESPONSE:
                print(RESPONSE[I])
        else:
            VAULT = get_VAULTclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = VAULT.get_cached_vaults(profile_name)
            show_as_table(RESPONSE)
    else:
        VAULT = get_VAULTclient(profile_name, auto_refresh=False, cache_only=True)
        RESPONSE = VAULT.describe(vault)
        Log.info(f"describing {secret}:\n" + json.dumps(RESPONSE, indent=2, sort_keys=True, default=str))

def show_as_table(source_data):
    IGNORE = ['last_cache_update']
    DATADICT = {}
    DATA = []
    try:
        for I in source_data:
            if I not in IGNORE:
                DATA.append(source_data[I])
                if DATA:
                    DATADICT = DATA
        if DATADICT != []:
            Log.info(f"\n{tabulate(DATADICT, headers='keys', tablefmt='rst')}\n")
    except:
        return None

def get_VAULTclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = VAULTclient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'Vault Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE

def get_region(ctx, profile_name):
    OCI_REGION = ctx.obj['REGION']
    if not OCI_REGION:
        VAULT = get_VAULTclient(profile_name)
        OCI_REGION = VAULT.get_region_from_profile(profile_name)
    return OCI_REGION

vault.add_command(keys)
vault.add_command(secrets)



import click, os, subprocess, tempfile, re
from toolbox.logger import Log
from .oci_config import OCIconfig
from . import iam_nongc
from configstore.configstore import Config
from toolbox.misc import set_terminal_width, detect_environment

CONFIG = Config('ocitools')
LATEST = CONFIG.get_profile('latest')

if LATEST is None:
    LATEST = 'DEFAULT'
else:
    LATEST = LATEST['config']['name']

CONTEXT_SETTINGS = {'help_option_names':['-h'], 'max_content_width': set_terminal_width(), 'ignore_unknown_options': True, 'allow_extra_args': True}

@click.command(help="run any ocicli (oci) command while leveraging ocitools profile functionality; append --help for usage on cli commands", context_settings=CONTEXT_SETTINGS)
@click.argument('ocicli_command', nargs=-1, type=str, required=False)
@click.option('-r', '--region', 'region', help='OCI region to authenticate against', required=False, default='us-ashburn-1', type=str)
@click.option('-i', '--interactive', 'interactive', help='spawn the interactive shell', is_flag=True, required=False, default=False)
@click.pass_context
def cli(ctx, ocicli_command, region, interactive):
    profile_name = ctx.obj['PROFILE']
    OCI = get_OCIcli(profile_name, region)
    CMD = ' '.join(ocicli_command) # convert tuple to string
    if interactive is True:
        try:
            os.system(f"oci --profile {profile_name} --region {region} -i")
        except:
            Log.critical(f'unable to launch oci -i for interactive use; please install the latest OCI CLI.')
    elif not CMD:
        try:
            os.system(f"oci --profile {profile_name} --region {region} -i")
        except:
            Log.critical(f'unable to launch oci -i for interactive use; please install the latest OCI CLI.')
    else:
        RESULT = OCI.run_cmd(f"oci --profile {profile_name} --region {region} {CMD}")
        print(RESULT)

class OCIcli():
    def __init__(self, profile_name, region='us-ashburn-1'):
        self.OCI_PROFILE = profile_name
        self.OCI_REGION = region
        self.CONFIG = OCIconfig()

    def run_cmd(self, cmd):
        RESULT = run_command(cmd)
        return RESULT

def get_OCIcli(profile_name, region='us-ashburn-1'):
    return OCIcli(profile_name, region)

def run_command(command):
    TEMP = tempfile.NamedTemporaryFile(delete=False)
    TEMP.close()
    PATH = TEMP.name
    CMD = command + " 1> " + PATH + " 2>" + PATH
    PROC = subprocess.run(CMD, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    with open(PATH, 'r') as f:
        DATA = f.read()
    os.unlink(PATH)
    return DATA
    



import click, os, json
from toolbox.logger import Log
from .keysclient import KEYclient
from .vaultclient import VAULTclient
from tabulate import tabulate
from .misc import get_random_password
from configstore.configstore import Config
from toolbox.misc import set_terminal_width, decode_string
from .iam import get_latest_profile
from toolbox.menumaker import Menu
from datetime import datetime
from datetime import timedelta

CONFIG = Config('ocitools')
IGNORE = ['latest']

@click.group('keys', invoke_without_command=True, help='module to manage vault keys', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform key actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def keys(ctx, menu):
    profile_name = ctx.obj['PROFILE']
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False
    pass

@keys.command(help='manually refresh keys stored in cache', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def refresh(ctx):
    profile_name = ctx.obj['PROFILE']
    _refresh('cached_keys', profile_name)

def _refresh(cache_type, profile_name):
    try:
        if cache_type == 'cached_keys':
            KEYS = get_KEYclient(profile_name, auto_refresh=False)
            RESULT = KEYS.refresh('cached_keys', profile_name)
        return True
    except:
        False

@keys.command(help='create OCI keys for region and tenant', context_settings={'help_option_names':['-h','--help']})
@click.option('-n', '--name', help='secret name', default=None, required=True, type=str)
@click.option('-d', '--description', help='short description for the secret; default uses the name of the secret as the description', default=None, required=False, type=str)
@click.pass_context
def create(ctx, name, description):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _create(ctx, profile_name, oci_region, name, description)

def _create(ctx, profile_name, oci_region, name, description):
    if description is None:
        description = name
    VAULT = get_VAULTclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    KEYS = get_KEYclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in IGNORE:
            continue
        CACHED_COMPARTMENTS = VAULT.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        COMP_OCID = CHOICE.split('\t')[0].strip()
        COMP_NAME = CHOICE.split('\t')[1].strip()
    else:
        Log.critical('please choose a compartment')
    VAULTS = VAULT.get_vaults(COMP_OCID)
    DATA = []
    for MYVAULT in VAULTS:
        if MYVAULT.lifecycle_state not in 'ACTIVE':
            continue
        VAULT_NAME = MYVAULT.display_name.ljust(25)
        VAULT_OCID = MYVAULT.id.ljust(50)
        ENDPOINT = MYVAULT.management_endpoint
        DATA.append(VAULT_OCID + '\t' + VAULT_NAME + '\t' + ENDPOINT)
    INPUT = f'Vaults => {profile_name}'
    CHOICE = runMenu(DATA, INPUT) 
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        VAULT_OCID = CHOICE.split('\t')[0].strip()
        VAULT_NAME = CHOICE.split('\t')[1].strip()
        ENDPOINT = CHOICE.split('\t')[2].strip()
    else:
        Log.critical('please choose a vault to create a key')
    DATA = []
    Log.info(f'creating AES encrypted key, please wait...')
    RESPONSE = KEYS.create_key(name, COMP_OCID, ENDPOINT)
    if RESPONSE:
        print(RESPONSE)
    else:
        Log.critical(f'problem while creating key {name}')

@keys.command(help='menu-guided solution to delete OCI keys for region and tenant', context_settings={'help_option_names':['-h','--help']})
@click.option('-a', '--all', help='delete all keys found', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def delete(ctx, all):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _delete(ctx, profile_name, oci_region, all)

def _delete(ctx, profile_name, oci_region, all):
    VAULT = get_VAULTclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    KEYS = get_KEYclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in IGNORE:
            continue
        CACHED_COMPARTMENTS = VAULT.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        COMP_OCID = CHOICE.split('\t')[0].strip()
        COMP_NAME = CHOICE.split('\t')[1].strip()
    else:
        Log.critical('please choose a compartment')
    VAULTS = VAULT.get_vaults(COMP_OCID)
    DATA = []
    for MYVAULT in VAULTS:
        if MYVAULT.lifecycle_state not in 'ACTIVE':
            continue
        VAULT_NAME = MYVAULT.display_name.ljust(25)
        VAULT_OCID = MYVAULT.id.ljust(50)
        ENDPOINT = MYVAULT.management_endpoint
        DATA.append(VAULT_OCID + '\t' + VAULT_NAME + '\t' + ENDPOINT)
    INPUT = f'Vaults => {profile_name}'
    CHOICE = runMenu(DATA, INPUT) 
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        VAULT_OCID = CHOICE.split('\t')[0].strip()
        VAULT_NAME = CHOICE.split('\t')[1].strip()
        ENDPOINT = CHOICE.split('\t')[2].strip()
    else:
        Log.critical('please choose a vault to search for keys')
    
    DAYS = datetime.now() + timedelta(days=15)
    RESULT = KEYS.list_keys(COMP_OCID, ENDPOINT)
    if all is not True:
        DATA = []
        for D in RESULT:
            ID = D.id.ljust(50)
            NAME = D.display_name.ljust(20)
            STATE = D.lifecycle_state
            DATA.append(ID + '\t' + NAME + '\t' + STATE)
        INPUT = f'Keys => {profile_name}'
        CHOICE = runMenu(DATA, INPUT) 
        if CHOICE:
            CHOICE = ''.join(CHOICE)
            KEY_OCID = CHOICE.split('\t')[0].strip()
            KEY_NAME = CHOICE.split('\t')[1].strip()
            STATE = CHOICE.split('\t')[2].strip()
        else:
            Log.critical('please choose a key to delete')
        Log.info(f'scheduling deletion of key {KEY_NAME} in compartment {COMP_NAME} for vault {VAULT_NAME}')
        RESPONSE = KEYS.schedule_deletion_key(DAYS, KEY_OCID, ENDPOINT)
        if RESPONSE:
            Log.info(f'{KEY_NAME} is now pending deletion on {DAYS}')
        else:
            Log.critical(f'problem while scheduling the deletion of {KEY_NAME}')
    else:
        for D in RESULT:
            ID = D.id
            NAME = D.display_name
            STATE = D.lifecycle_state
            if STATE != 'PENDING_DELETION':
                Log.info(f'scheduling deletion of key {NAME} in compartment {COMP_NAME} for vault {VAULT_NAME}')
                RESPONSE = KEYS.schedule_deletion_key(DAYS, ID, ENDPOINT)
                if RESPONSE:
                    Log.info(f'{NAME} is now pending deletion on {DAYS}')
                else:
                    Log.critical(f'problem while scheduling the deletion of {NAME}')

@keys.command(help='show the keys stored in cache', context_settings={'help_option_names':['-h','--help']})
@click.argument('key', required=False)
@click.pass_context
def show(ctx, key):
    profile_name = ctx.obj['PROFILE']
    _show(ctx, profile_name, key)

def _show(ctx, profile_name, key):
    if not key:
        if ctx.obj["MENU"]:
            DATA = []
            DICT = {}
            KEYS = get_KEYclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = KEYS.get_cached_keys(profile_name)
            for I in RESPONSE:
                if I not in 'last_cache_update':
                    NAME = RESPONSE[I]['secret_name']
                    ID = I.ljust(50)
                    DATA.append(ID + '\t' + NAME)
            INPUT = 'Key Manager'
            CHOICE = runMenu(DATA, INPUT)
            try:
                CHOICE = ''.join(CHOICE)
                OCID = CHOICE.split('\t')[0].strip()
            except:
                Log.critical("please select a key to continue...")
            RESPONSE = KEYS.describe(OCID, profile_name)
            DATA = {}
            Log.info(f"describing {OCID}:")
            for I in RESPONSE:
                print(RESPONSE[I])
        else:
            KEYS = get_KEYclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = KEYS.get_cached_keys(profile_name)
            show_as_table(RESPONSE)
    else:
        KEYS = get_KEYclient(profile_name, auto_refresh=False, cache_only=True)
        RESPONSE = KEYS.describe(key, profile_name)
        Log.info(f"describing {key}:\n" + json.dumps(RESPONSE, indent=2, sort_keys=True, default=str))

def show_as_table(source_data):
    IGNORE = ['last_cache_update']
    DATADICT = {}
    DATA = []
    try:
        for I in source_data:
            if I not in IGNORE:
                DATA.append(source_data[I])
                if DATA:
                    DATADICT = DATA
        if DATADICT != []:
            Log.info(f"\n{tabulate(DATADICT, headers='keys', tablefmt='rst')}\n")
    except:
        return None

def get_KEYclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = KEYclient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def get_VAULTclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = VAULTclient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'Key Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE

def get_region(ctx, profile_name):
    OCI_REGION = ctx.obj['REGION']
    if not OCI_REGION:
        VAULT = get_VAULTclient(profile_name)
        OCI_REGION = VAULT.get_region_from_profile(profile_name)
    return OCI_REGION



#!/usr/bin/env python3

# script compiled from oci oracle examples

import oci
import sys
import os.path
from hashlib import md5
from codecs import decode
from time import sleep

if len(sys.argv) != 3:
    print("INFO: all parameters are required to run this tool:")
    print("- 1st param = public key location")
    print("- 2nd param = region")
    sys.exit()

public_key_path = sys.argv[1]
region = sys.argv[2]

config = oci.config.from_file(profile_name='default')

def generate_signer_from_config(config):
    signer = oci.signer.Signer(
        tenancy=config["tenancy"],
        user=config["user"],
        fingerprint=config["fingerprint"],
        private_key_file_location=config.get("key_file")
    )

    return signer

signer = generate_signer_from_config(config)
identity = oci.identity.IdentityClient(config, signer=signer)
compute_client = oci.core.ComputeClient(config, signer=signer)
virtual_network_client = oci.core.VirtualNetworkClient(config, signer=signer)

def get_fingerprint(key):

    # There should be more error checking here, but to keep the example simple
    # the error checking has been omitted.
    m = md5()

    # Strip out the parts of the key that are not used in the fingerprint
    # computation.
    key = key.replace(b'-----BEGIN PUBLIC KEY-----\n', b'')
    key = key.replace(b'\n-----END PUBLIC KEY-----', b'')

    # The key is base64 encoded and needs to be decoded before getting the md5
    # hash
    decoded_key = decode(key, "base64")
    m.update(decoded_key)
    hash = m.hexdigest()

    # Break the hash into 2 character parts.
    length = 2
    parts = list(hash[0 + i:length + i] for i in range(0, len(hash), length))

    # Join the parts with a colon seperator
    fingerprint = ":".join(parts)
    return fingerprint

def is_key_already_uploaded(keys, fingerprint):
    for key in keys:
        if key.fingerprint == fingerprint:
            return True
    return False

# Check there are enough arguments
if len(sys.argv) != 3:
    raise RuntimeError('INFO: this script expects an argument of a path to the public API key')

# Verify the path to the public key exists
if not os.path.exists(public_key_path):
    raise RuntimeError('INFO: this argument must be a valid path to a public API key')

# Open the key file and store the contents
with open(public_key_path, 'rb') as f:
    public_key = f.read().strip()

# Get the fingerprint for the key
fingerprint = get_fingerprint(public_key)
print("INFO: fingerprint of API key is: {}".format(fingerprint))

# Read the config file and initialize the identity client
config = oci.config.from_file(profile_name='default')

# Check to see if this key is already associated with the user.
if is_key_already_uploaded(identity.list_api_keys(config['user']).data, fingerprint):
    print("INFO: key with fingerprint {} has already been added to user".format(fingerprint))
    sys.exit()

# Initialize the CreateApiKeyDetails model
key_details = oci.identity.models.CreateApiKeyDetails(key=public_key.decode())

# Upload the key
response = identity.upload_api_key(config['user'], key_details)

# Results of uploading key
print("INFO: results of uploading key:")
print()
print(response.data)
print()

while True:
    if is_key_already_uploaded(identity.list_api_keys(config['user']).data, fingerprint):
        print("INFO: sucessfully uploaded API key {}".format(public_key_path))
        break
    sleep(2)



#!/usr/bin/env python
# ==============================================================================
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
# ==============================================================================

import sys, os
c = '/opt/native/gen2cli/.override_colors'

RESET = '\033[0m'
CYAN = '\033[0;36m'
RED = '\033[31m'
boldRED = '\033[0;31m'
YELLOW = '\033[0;33m'
GREEN = '\033[0;32m'
MAGENTA = '\033[0;35m'
MYBLUE = '\033[0;2;60m'
BLUE = '\033[38;5;69m'
GREY = '\033[0;49;39m'
LIGHTGREY = '\033[0;37m'
ORANGE = '\033[38;5;172m'
UNDERLINE = '\033[4m'
BOLD = '\033[1m'
SCREEN_WIDTH = 95
MOVE1 = '\033[70G'
MOVE2 = '\033[100G'
WHITE = '\033[1;37m'

if os.path.isfile(c):
    COLOR = (([line.split('=')[1].strip('\n').strip('"') for line in open(c) if 'USAGE_COLOR' in line]))
    COLOR = ''.join(map(str, COLOR))
    if 'GREEN' in COLOR:
        USAGE_COLOR = GREEN
    elif 'CYAN' in COLOR:
        USAGE_COLOR = CYAN
    elif 'MYBLUE' in COLOR:
        USAGE_COLOR = MYBLUE
    elif 'RED' in COLOR:
        USAGE_COLOR = RED
    elif 'MAGENTA' in COLOR:
        USAGE_COLOR = MAGENTA
    elif 'YELLOW' in COLOR:
        USAGE_COLOR = YELLOW
    elif 'ORANGE' in COLOR:
        USAGE_COLOR = ORANGE
    elif 'BLUE' in COLOR:
        USAGE_COLOR = BLUE
    elif 'LIGHTGREY' in COLOR:
        USAGE_COLOR = LIGHTGREY
    elif 'GREY' in COLOR:
        USAGE_COLOR = GREY
    else:
        USAGE_COLOR = MYBLUE
else:
    USAGE_COLOR = MYBLUE

def get_username():
    from os import environ, getcwd
    user = (lambda: environ["USERNAME"] if "C:" in getcwd() else environ["USER"])()
    return user



import click, string, random, subprocess, oci, os
from .oci_config import OCIconfig
from oci.config import from_file
from toolbox.logger import Log
from os import environ, chmod, stat
from configstore.configstore import Config
from toolbox.misc import set_terminal_width

CONFIG = OCIconfig()
STORE = Config('ocitools')

@click.command(help='login using existing IAM creds or add new creds to config', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-r', '--region', 'region', help='oci region to connect to', required=False, default='us-ashburn-1')
@click.option('-t', '--tenant', 'tenant', help='tenant ocid to connect to', required=False, default=None)
@click.option('-u', '--user', 'user', help='user ocid to connect with', required=False, default=None)
@click.option('-k', '--keyfile', 'keyfile', help='user oci API private key file to connect with', required=False, default="~/.oci/oci_api_key.pem")
@click.option('-f', '--fingerprint', 'fingerprint', help='fingerprint of oci private key to connect with', required=False, default=None)
@click.option('-p', '--profile', 'profile', help='oci profile to connect with', required=False, default='DEFAULT')
@click.pass_context
def authenticate(ctx, region, tenant, user, keyfile, fingerprint, profile):
    if profile is None:
        profile_name = ctx.obj['PROFILE']
    else:
        profile_name = str(profile)
        ctx.obj['PROFILE'] = profile_name
    RESULT = _authenticate(tenant, user, fingerprint, keyfile, profile_name, region)
    if RESULT is None:
        MSG = 'setup failed. please verify supplied credentials and try again. settings were not saved'
        LINK = 'https://github.com/stacksc/goat'
        CMD = None
        SUBTITLE = 'CRITICAL'
        TITLE = 'GOAT'
        Log.notify(MSG, TITLE, SUBTITLE, LINK, CMD)
        Log.critical(MSG)
    else:
        Log.info("credentials saved successfully")
        if profile_name not in STORE.PROFILES:
            STORE.create_profile(profile_name)
        # update metadata facts
        try:
            STORE.update_metadata(fingerprint, 'fingerprint', profile_name, True)
            STORE.update_metadata(keyfile, 'key_file', profile_name, True)
            STORE.update_metadata(tenant, 'tenancy', profile_name, True)
            STORE.update_metadata(region, 'region', profile_name, True)
            STORE.update_metadata(user, 'user', profile_name, True)
        except:
            Log.critical('unable to update the configstore with OCI information')
        Log.info(f"you can now use your new profile with 'oci --profile {profile_name}")
        MSG = f'{profile_name} credentials saved successfully!'
        LINK = None
        CMD = None
        SUBTITLE = 'INFO'
        TITLE = 'GOAT'
        Log.notify(MSG, TITLE, SUBTITLE, LINK, CMD)
        cache_all_hack(profile_name, region)
        update_latest_profile(profile_name)

# worker function to make the method portable
def _authenticate(tenant, user, fingerprint, keyfile, profile_name, region='us-ashburn-1'):
    # Check if parameters were provided
    if tenant is None or user is None or fingerprint is None or keyfile is None:
        try:
            # Attempt to read from OCI config file
            config = from_file("~/.oci/config", profile_name)
            tenant = config.get("tenancy")
            user = config.get("user")
            fingerprint = config.get("fingerprint")
            keyfile = config.get("key_file")
        except Exception as e:
            print(f"Error reading OCI config file: {e}")
            return None

    CONFIG.add_oci_profile(tenant, user, region, fingerprint, keyfile, profile_name)
    try:
        SESSION = oci.config.from_file("~/.oci/config", profile_name)
    except:
        Log.warn('unable to load the new config file')
        return None
    return SESSION

def update_latest_profile(profile_name):
    BASICS = Config('ocitools')
    LATEST = BASICS.get_profile('latest')
    if LATEST is None:
        BASICS.create_profile('latest')
        BASICS.update_config(profile_name, 'name', 'latest')
    else:
        BASICS.update_config(profile_name, 'name', 'latest')

def listToStringWithoutBrackets(list1):
    return str(list1).replace('[','').replace(']','').replace("'", "")

def force_cache(profile_name, region):
    CONFIG = Config('ocitools')
    MODULES = ['oss', 'compute', 'dbs', 'regions', 'vault', 'secrets', 'compartment', 'keys']
    for MODULE in MODULES:
        if MODULE == 'secrets' or MODULE == 'keys':
            os.system(f'goat oci -r {region} -p {profile_name} vault {MODULE} refresh')
        else:
            try:
                os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            except:
                pass

def cache_all_hack(profile_name, region):
    CONFIG = Config('ocitools')
    MODULES = ['oss', 'compute', 'dbs', 'regions', 'vault', 'secrets', 'compartment', 'keys']
    for MODULE in MODULES:
        if MODULE == 'oss':
            CACHED = {}
            FOUND = False
            try:
                CACHED.update(CONFIG.get_metadata('cached_buckets', profile_name))
            except:
                pass
            if not CACHED: 
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'vault':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.get_metadata('cached_vaults', profile_name))
            except:
                pass
            if not CACHED: 
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'compartment':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.get_metadata('cached_compartments', profile_name))
            except:
                pass
            if not CACHED: 
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'secrets':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.get_metadata('cached_secrets', profile_name))
            except:
                pass
            if not CACHED: 
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} vault {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} vault {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'keys':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.get_metadata('cached_keys', profile_name))
            except:
                pass
            if not CACHED: 
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} vault {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} vault {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'regions':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.get_metadata('cached_regions', profile_name))
            except:
                pass
            if not CACHED: 
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'compute':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.get_metadata('cached_instances', profile_name))
            except:
                pass
            if not CACHED:
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')
        elif MODULE == 'dbs':
            FOUND = False
            CACHED = {}
            try:
                CACHED.update(CONFIG.PROFILES[profile_name]['metadata']['cached_dbs_instances'])
            except:
                pass
            if not CACHED:
                try:
                    os.system(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
                except:
                    pass
            elif CACHED:
                for REGION in CACHED:
                    if region == REGION:
                        FOUND = True
                        break
                if FOUND is False:
                    run_command(f'goat oci -r {region} -p {profile_name} {MODULE} refresh')
            else:
                Log.info(f'cache exists for {MODULE} data...')

def run_command(command):
    PROC = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)




import oci
from os import environ, unsetenv, mkdir, path
from pathlib import Path
from configparser import ConfigParser
from toolbox.logger import Log
from toolbox.getpass import get_secure_string

class OCIconfig():
    def __init__(self, oci_dir=f"{environ['HOME']}/.oci/", oci_config="config"):
        self.OCI_DIR = oci_dir
        self.OCI_CONFIG_FILE = self.verify_path(f"{oci_dir}{oci_config}")

    def add_oci_profile(self, tenancy, user, region, fingerprint, key_file, profile_name):
        self.update_oci_config('config', profile_name, 'region', str(region))
        self.update_oci_config('config', profile_name, 'tenancy', str(tenancy))
        self.update_oci_config('config', profile_name, 'user', str(user))
        self.update_oci_config('config', profile_name, 'fingerprint', str(fingerprint))
        self.update_oci_config('config', profile_name, 'key_file', str(key_file))

    def update_oci_config(self, config_type, profile_name, property_name, property_value):
        CONFIG, CONFIG_FILE = self.choose_configparser(config_type)
        try:
            CONFIG.add_section(profile_name)
        except:
            pass # this means the section already exists
        if property_value is not None:  # Only set non-None values
            CONFIG.set(profile_name, property_name, str(property_value))
        with open(CONFIG_FILE, 'w') as CONFIG_FILE_STREAM:
            CONFIG.write(CONFIG_FILE_STREAM) 

    def get_from_config(self, config_type, config_property_name, alternative_method=None, alternative_method_value=None, profile_name=None, supplied_value=None):
        CONFIG, CONFIG_FILE = self.choose_configparser(config_type)
        if profile_name in CONFIG and config_property_name in CONFIG[profile_name]:
            RESULT = CONFIG[profile_name][config_property_name]
        else:
            RESULT = None
        if RESULT is None:
            if alternative_method == 'string':
                RESULT = alternative_method_value
            elif alternative_method == 'input':
                RESULT = input(alternative_method_value)
            elif alternative_method == 'secure':
                RESULT = get_secure_string(config_property_name, alternative_method_value)
            else:
                pass
        if supplied_value != None:
            if RESULT != supplied_value:
                RESULT = supplied_value
        return RESULT

    def choose_configparser(self, config_type):
        CONFIG = ConfigParser() 
        if config_type == "config":
            CONFIG_FILE = self.OCI_CONFIG_FILE
        else:
            Log.critical("invalid config type")
        CONFIG.read(CONFIG_FILE)
        return CONFIG, CONFIG_FILE

    def verify_path(self, path_to_file):
        try:
            PATH_TO_FOLDER = '/'.join(path_to_file.split('/')[:-1])
            if not path.exists(PATH_TO_FOLDER):
                mkdir(PATH_TO_FOLDER)
            Path(path_to_file).touch()
            return path_to_file
        except OSError:
            Log.critical("failed to create .oci directory")

    def display(self, type, profile_name=None):
        OCI_CONFIG, OCI_FILE = self.choose_configparser(type)
        if profile_name is None:
            print(f'\n-------------------- oci {type} for all profiles --------------------')
            for PROFILE in OCI_CONFIG:
                if PROFILE != 'DEFAULT': # oci uses uppercase default, remove redundant DEFAULT entry from output
                    print(f"    [{PROFILE}]")
                for PROPERTY in OCI_CONFIG[PROFILE]:
                    print(f"    {PROPERTY}={OCI_CONFIG[PROFILE][PROPERTY]}")
                print('')
            print('')
        else:
            if profile_name in OCI_CONFIG:
                print(f'\n-------------------- oci {type} for profile {profile_name} --------------------')
                print(f"    [{profile_name}]")
                for PROPERTY in OCI_CONFIG[profile_name]:
                        print(f"    {PROPERTY}={OCI_CONFIG[profile_name][PROPERTY]}")
                print('')
            else:
                return



import click
from configstore.configstore_ctrl import _show as configshow
from .oci_config import OCIconfig
from .iam import get_latest_profile

@click.command(help='display configuration data for ocitools and ocicli')
@click.argument('target', required=True, type=click.Choice(['all_config', 'oci_config' ]))
@click.pass_context
def show(ctx, target):
    profile_name = ctx.obj['PROFILE'].lower()
    if profile_name is None:
        profile_name = 'DEFAULT'
    if target == 'all_config':
        configshow('ocitools', profile_name)
    if target == 'oci_config':
        OCI_CONFIG = OCIconfig()
        OCI_CONFIG.display('config', profile_name)
